// (c) Peter Kankowski, 2009 http://www.strchr.com kankowski@narod.ru
#include <iostream>
#include <iomanip>
#include <windows.h>

// Class for reading a small file as a whole.
// Under 32-bit Windows, the file should be less than 2 Gb
// minus memory used by your application.
class FileReader
{
private:
	HANDLE m_file;
	HANDLE m_mapping;
	const char * m_filedata;
	const char * m_content;
	DWORD m_filesize;

public:
	FileReader(const TCHAR * filename) :
	  m_file(NULL), m_mapping(NULL), m_filedata(NULL), m_content(NULL), m_filesize(0)
	{
		m_file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if(INVALID_HANDLE_VALUE == m_file)
			return;
		
		m_filesize = GetFileSize(m_file, NULL);
		if(INVALID_FILE_SIZE == m_filesize)
			return;
		
		// Skip zero-length files, because MMF fails for them
		if(0 == m_filesize) {
			m_content = "";
			return;
		}
		
		m_mapping = CreateFileMapping(m_file, NULL, PAGE_READONLY, 0, 0, NULL);
		if(!m_mapping)
			return;

		m_content = m_filedata = (const char *)MapViewOfFile(m_mapping,
			FILE_MAP_READ, 0, 0, 0);
	}
	const char * get_content() const
	{
		return m_content;
	}
	DWORD get_size() const
	{
		return m_filesize;
	}
	~FileReader()
	{
		UnmapViewOfFile((void *)m_filedata);
		CloseHandle(m_mapping);
		CloseHandle(m_file);
	}
};

void display_win32_error()
{
	TCHAR * errormsg;
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&errormsg, 0, NULL);
	std::cout << errormsg;
	LocalFree(errormsg);
}

int main(void)
{
	// Usage example: read and print a small file
	FileReader f( TEXT("C://file.txt") );
	if (!f.get_content()) {
		display_win32_error();
		return 1;
	}
	std::cout << std::setw(f.get_size()) << f.get_content();
	return 0;
}
